Package: mosaic
Type: Package
Title: Project MOSAIC Statistics and Mathematics Teaching Utilities
Version: 1.10.2
Description: Data sets and utilities from Project MOSAIC (<http://www.mosaic-web.org>) used
    to teach mathematics, statistics, computation and modeling.  Funded by the
    NSF, Project MOSAIC is a community of educators working to tie together
    aspects of quantitative work that students in science, technology,
    engineering and mathematics will need in their professional lives, but
    which are usually taught in isolation, if at all.
Authors@R: c(
    person(given = "Randall",  family = "Pruim", email = "rpruim@gmail.com", role = c("aut", "cre")), 
    person(given = "Daniel T.",   family = "Kaplan", email = "dtkaplan@macalester.edu", role = "aut"), 
    person(given = "Nicholas J.", family = "Horton", email = "nhorton@amherst.edu", role = "aut")
    )
Depends: R (>= 4.1),
Imports: dplyr, tibble, lattice (>= 0.20-21), ggformula, mosaicData,
        Matrix, mosaicCore (>= 0.7.0), ggplot2, rlang (>= 0.4.7),
        purrr, MASS, grid, tidyr, methods, utils
Suggests: ggstance, ggridges, vdiffr, lubridate, magrittr, NHANES,
        RCurl, sp, vcd, testthat (>= 3.0.0), knitr, tools, parallel,
        mapproj, rgl, rmarkdown, covr, formatR, palmerpenguins,
        ggrepel, readr, ggdendro, gridExtra, splines, latticeExtra,
        glue, broom, leaflet
Enhances: manipulate
VignetteBuilder: knitr
License: GPL (>= 2)
LazyLoad: yes
LazyData: yes
URL: https://github.com/ProjectMOSAIC/mosaic,
        https://www.mosaic-web.org/mosaic/
BugReports: https://github.com/ProjectMOSAIC/mosaic/issues
Encoding: UTF-8
Config/testthat/edition: 3
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-05-13 01:53:14 UTC; rpruim
Author: Randall Pruim [aut, cre],
  Daniel T. Kaplan [aut],
  Nicholas J. Horton [aut]
Maintainer: Randall Pruim <rpruim@gmail.com>
Repository: CRAN
Date/Publication: 2026-05-13 07:30:21 UTC
Built: R 4.6.0; ; 2026-05-13 08:56:43 UTC; unix
