# Generated by roxygen2: do not edit by hand

S3method(TukeyHSD,formula)
S3method(TukeyHSD,lm)
S3method(as.xtabs,data.frame)
S3method(as.xtabs,matrix)
S3method(boott,do.data.frame)
S3method(chisq,default)
S3method(chisq,htest)
S3method(chisq,table)
S3method(coef,nlsfunction)
S3method(confint,data.frame)
S3method(confint,do.data.frame)
S3method(confint,do.tbl_df)
S3method(confint,htest)
S3method(confint,numeric)
S3method(confint,summary.lm)
S3method(cor_test,default)
S3method(cor_test,formula)
S3method(cull_for_do,aggregated.stat)
S3method(cull_for_do,anova)
S3method(cull_for_do,aov)
S3method(cull_for_do,cointoss)
S3method(cull_for_do,default)
S3method(cull_for_do,fitdistr)
S3method(cull_for_do,htest)
S3method(cull_for_do,lm)
S3method(cull_for_do,lme)
S3method(cull_for_do,matrix)
S3method(cull_for_do,table)
S3method(do,default)
S3method(do,numeric)
S3method(factorize,character)
S3method(factorize,data.frame)
S3method(factorize,default)
S3method(factorize,numeric)
S3method(fortify,SpatialPolygonsDataFrame)
S3method(fortify,TukeyHSD)
S3method(fortify,hclust)
S3method(fortify,summary.glm)
S3method(fortify,summary.lm)
S3method(model,nlsfunction)
S3method(mplot,TukeyHSD)
S3method(mplot,data.frame)
S3method(mplot,default)
S3method(mplot,hclust)
S3method(mplot,lm)
S3method(mplot,summary.glm)
S3method(mplot,summary.lm)
S3method(plot,freqpolygon)
S3method(plotCumfreq,default)
S3method(plotCumfreq,formula)
S3method(plotModel,default)
S3method(plotModel,parsedModel)
S3method(print,cointoss)
S3method(print,oddsRatio)
S3method(print,relrisk)
S3method(print,repeater)
S3method(prop_test,character)
S3method(prop_test,default)
S3method(prop_test,factor)
S3method(prop_test,formula)
S3method(prop_test,logical)
S3method(prop_test,numeric)
S3method(pval,htest)
S3method(rescale,character)
S3method(rescale,data.frame)
S3method(rescale,default)
S3method(rescale,factor)
S3method(rescale,numeric)
S3method(sample,data.frame)
S3method(sample,default)
S3method(sample,factor)
S3method(sample,lm)
S3method(sample,matrix)
S3method(solve,formula)
S3method(stat,htest)
S3method(stat,uneval)
S3method(summary,nlsfunction)
S3method(summary,oddsRatio)
S3method(summary,relrisk)
S3method(t_test,default)
S3method(t_test,formula)
S3method(value,default)
S3method(value,integrate)
export(Broyden)
export(CIAdata)
export(CIsim)
export(Do)
export(IQR)
export(MAD)
export(MAD_)
export(SAD)
export(SAD_)
export(adapt_seq)
export(aggregatingFunction1)
export(aggregatingFunction1or2)
export(aggregatingFunction2)
export(as.xtabs)
export(ashplot)
export(bargraph)
export(binom.test)
export(cdata)
export(cdata_f)
export(cdata_v)
export(cdist)
export(chisq)
export(cnorm)
export(col.mosaic)
export(compare)
export(compareMean)
export(compareProportion)
export(condition)
export(connector)
export(cor)
export(cor.test)
export(cor_test)
export(count)
export(counts)
export(cov)
export(cross)
export(ct)
export(cull_for_do)
export(ddata)
export(ddata_f)
export(ddata_v)
export(deal)
export(deg2rad)
export(deltaMethod)
export(derivedFactor)
export(derivedVariable)
export(design_plot)
export(dfapply)
export(diffmean)
export(diffprop)
export(do)
export(docFile)
export(dot)
export(dotPlot)
export(dpqrdist)
export(ediff)
export(expandFun)
export(factorise)
export(factorize)
export(fav_stats)
export(favstats)
export(fetchData)
export(fetchGapminder)
export(fetchGapminder1)
export(fetchGoogle)
export(findZeros)
export(findZerosMult)
export(fitModel)
export(fitSpline)
export(fitdistr)
export(fivenum)
export(fractions)
export(freqpoly)
export(freqpolygon)
export(getVarFormula)
export(googleMap)
export(gwm)
export(hist2freqpolygon)
export(ilogit)
export(inferArgs)
export(inspect)
export(iqr)
export(is.wholenumber)
export(ladd)
export(latlon2xyz)
export(leaflet_map)
export(lhs)
export(linearModel)
export(logit)
export(lonlat2xyz)
export(mMap)
export(mPlot)
export(mScatter)
export(mUSMap)
export(mUniplot)
export(mWorldMap)
export(maggregate)
export(makeColorscheme)
export(makeFun)
export(makeMap)
export(mat)
export(max)
export(mean)
export(mean_)
export(median)
export(mid)
export(min)
export(mm)
export(model)
export(mosaic.getOption)
export(mosaic.options)
export(mosaic.par.get)
export(mosaic.par.set)
export(mosaicLatticeOptions)
export(mplot)
export(msummary)
export(n_missing)
export(nflip)
export(ntiles)
export(oddsRatio)
export(orrr)
export(panel.ashplot)
export(panel.cumfreq)
export(panel.dotPlot)
export(panel.freqpolygon)
export(panel.levelcontourplot)
export(panel.lmbands)
export(panel.plotFun)
export(panel.plotFun1)
export(panel.xhistogram)
export(panel.xqqmath)
export(pdata)
export(pdata_f)
export(pdata_v)
export(pdist)
export(perc)
export(percs)
export(perctable)
export(plotCumfreq)
export(plotDist)
export(plotFun)
export(plotModel)
export(plotPoints)
export(prepanel.cumfreq)
export(prepanel.default.ashplot)
export(prepanel.default.freqpolygon)
export(prepanel.xhistogram)
export(prod)
export(project)
export(prop)
export(prop.test)
export(prop1)
export(prop_test)
export(props)
export(proptable)
export(pval)
export(qdata)
export(qdata_f)
export(qdata_v)
export(qdist)
export(quantile)
export(r.squared)
export(rad2deg)
export(rand)
export(range)
export(rdata)
export(rdata_f)
export(rdata_v)
export(read.file)
export(relm)
export(relrisk)
export(resample)
export(rescale)
export(restoreLatticeOptions)
export(rflip)
export(rfun)
export(rgeo)
export(rgeo2)
export(rhs)
export(rlatlon)
export(rlonlat)
export(rpoly2)
export(rspin)
export(rsquared)
export(rstudio_is_available)
export(sample)
export(sd)
export(set.rseed)
export(shuffle)
export(singvals)
export(smoother)
export(sp2df)
export(spliner)
export(standardCountry)
export(standardName)
export(standardState)
export(stat)
export(statTally)
export(sum)
export(surround)
export(swap)
export(t.test)
export(t_test)
export(tally)
export(theme.mosaic)
export(theme_map)
export(update_ci)
export(value)
export(var)
export(vlength)
export(xcbeta)
export(xcbinom)
export(xcchisq)
export(xcf)
export(xcgamma)
export(xcgeom)
export(xchisq.test)
export(xcnbinom)
export(xcnorm)
export(xcpois)
export(xct)
export(xhistogram)
export(xhistogramBreaks)
export(xpbeta)
export(xpbinom)
export(xpchisq)
export(xpf)
export(xpgamma)
export(xpgeom)
export(xpnbinom)
export(xpnorm)
export(xppois)
export(xpt)
export(xqbeta)
export(xqbinom)
export(xqchisq)
export(xqf)
export(xqgamma)
export(xqgeom)
export(xqnbinom)
export(xqnorm)
export(xqpois)
export(xqqmath)
export(xqt)
export(xyz2latlon)
export(zscore)
exportClasses(repeater)
exportMethods("*")
exportMethods(project)
import(ggformula)
import(ggplot2)
import(lattice)
import(methods)
import(mosaicCore)
import(mosaicData)
import(rlang)
importFrom(MASS,fitdistr)
importFrom(MASS,fractions)
importFrom(Matrix,mean)
importFrom(dplyr,arrange)
importFrom(dplyr,bind_cols)
importFrom(dplyr,bind_rows)
importFrom(dplyr,do)
importFrom(dplyr,filter)
importFrom(dplyr,first)
importFrom(dplyr,group_by)
importFrom(dplyr,left_join)
importFrom(dplyr,mutate)
importFrom(dplyr,n)
importFrom(dplyr,select)
importFrom(dplyr,summarise)
importFrom(dplyr,ungroup)
importFrom(ggplot2,fortify)
importFrom(grDevices,col2rgb)
importFrom(grDevices,colorRampPalette)
importFrom(grDevices,gray)
importFrom(grDevices,heat.colors)
importFrom(grDevices,rgb)
importFrom(grDevices,topo.colors)
importFrom(graphics,hist)
importFrom(graphics,plot)
importFrom(graphics,stem)
importFrom(grid,gpar)
importFrom(grid,grid.layout)
importFrom(grid,grid.lines)
importFrom(grid,grid.points)
importFrom(grid,grid.polyline)
importFrom(grid,grid.rect)
importFrom(grid,grid.text)
importFrom(grid,unit.c)
importFrom(mosaicCore,compare)
importFrom(mosaicCore,condition)
importFrom(mosaicCore,count)
importFrom(mosaicCore,counts)
importFrom(mosaicCore,dfapply)
importFrom(mosaicCore,ediff)
importFrom(mosaicCore,ilogit)
importFrom(mosaicCore,inspect)
importFrom(mosaicCore,lhs)
importFrom(mosaicCore,logit)
importFrom(mosaicCore,makeFun)
importFrom(mosaicCore,mosaic_formula)
importFrom(mosaicCore,mosaic_formula_q)
importFrom(mosaicCore,msummary)
importFrom(mosaicCore,n_missing)
importFrom(mosaicCore,named)
importFrom(mosaicCore,named_among)
importFrom(mosaicCore,nice_names)
importFrom(mosaicCore,perc)
importFrom(mosaicCore,percs)
importFrom(mosaicCore,prop)
importFrom(mosaicCore,prop1)
importFrom(mosaicCore,props)
importFrom(mosaicCore,rhs)
importFrom(mosaicCore,tally)
importFrom(mosaicCore,unnamed)
importFrom(mosaicCore,vector2df)
importFrom(purrr,map_dbl)
importFrom(purrr,possibly)
importFrom(stats,TukeyHSD)
importFrom(stats,addmargins)
importFrom(stats,aggregate)
importFrom(stats,anova)
importFrom(stats,aov)
importFrom(stats,approxfun)
importFrom(stats,as.formula)
importFrom(stats,chisq.test)
importFrom(stats,coef)
importFrom(stats,complete.cases)
importFrom(stats,confint)
importFrom(stats,cutree)
importFrom(stats,dbeta)
importFrom(stats,dcauchy)
importFrom(stats,dchisq)
importFrom(stats,dexp)
importFrom(stats,df)
importFrom(stats,dgamma)
importFrom(stats,dist)
importFrom(stats,dlnorm)
importFrom(stats,dnorm)
importFrom(stats,dpois)
importFrom(stats,dt)
importFrom(stats,dweibull)
importFrom(stats,formula)
importFrom(stats,lm)
importFrom(stats,loess)
importFrom(stats,loess.control)
importFrom(stats,lsfit)
importFrom(stats,model.frame)
importFrom(stats,model.matrix)
importFrom(stats,model.response)
importFrom(stats,na.omit)
importFrom(stats,nls)
importFrom(stats,pnorm)
importFrom(stats,ppoints)
importFrom(stats,predict)
importFrom(stats,printCoefmat)
importFrom(stats,qnorm)
importFrom(stats,qqnorm)
importFrom(stats,qt)
importFrom(stats,rbinom)
importFrom(stats,reorder)
importFrom(stats,resid)
importFrom(stats,rmultinom)
importFrom(stats,rnorm)
importFrom(stats,runif)
importFrom(stats,setNames)
importFrom(stats,splinefun)
importFrom(stats,terms)
importFrom(stats,uniroot)
importFrom(stats,update)
importFrom(stats,vcov)
importFrom(stats,wilcox.test)
importFrom(stats,xtabs)
importFrom(tibble,rownames_to_column)
importFrom(tibble,tibble)
importFrom(tidyr,gather)
importFrom(utils,browseURL)
importFrom(utils,capture.output)
importFrom(utils,head)
importFrom(utils,menu)
importFrom(utils,modifyList)
importFrom(utils,packageVersion)
importFrom(utils,read.csv)
importFrom(utils,read.delim)
importFrom(utils,read.table)
importFrom(utils,str)
importFrom(utils,tail)
