NCSA Mosaic for the X Window System Version 2.4
===============================================

Welcome to NCSA Mosaic for the X Window System 2.4!

This README details installation steps.  

README-2.0 details major differences and new features in the 2.x
versions of Mosaic and should be read by anyone migrating from Mosaic
1.x.

More complete information and documentation on NCSA Mosaic is
available online, via NCSA Mosaic.


Binaries
--------

NCSA Mosaic is known to compile on the following platforms:

  SGI (IRIX 4.0.2) 
  SGI (IRIX 5.1.1) 
  IBM (AIX 3.2.4 with IBM X11R5)
  Sun 4 (SunOS 4.1.3 with stock X11R4 and Motif 1.1).
  DECstation 5000-200 (Ultrix).
  DEC Alpha (OSF/1).
  Hewlett Packard HP/UX 7.x, 8.x, 9.x.

Binaries for these platforms (and possibly others) are available on
ftp.ncsa.uiuc.edu in /Mosaic/Mosaic-binaries.

Rumor has it that Mosaic will compile on systems like:

  Dell SVR4
  Sun Solaris 2.x
  SCO Open Desktop
  ...lots others...
  
If you have to make nontrivial changes to NCSA Mosaic to get it to
compile on a particular platform, please send a set of context diffs
(e.g., 'diff -c oldfile newfile') to mosaic-x@ncsa.uiuc.edu.


Installation Instructions
-------------------------

Simply examine the toplevel Makefile, change the appropriate
customizable options, and type 'make'.  

The final result is a single independent executable, src/Mosaic.

(The Makefile.[sun,dec,ibm,alpha,etc.] files are the Makefiles we use
locally for compilation on various platforms; they will almost
certainly NOT WORK for you without modification.  We recommend you
start with the stock Makefile and make modifications as necessary to
avoid confusion.)

There is one tricky thing:

-- You have the option of compiling in support for NCSA HDF, a
   platform-independent hierarchical scientific data format, and NCSA
   DTM, a network-based message-passing protocol useful for exchanging
   scientific data between applications.  If you compile one of them
   in, you should compile both of them in.  

   If you don't already know what HDF and DTM are and want to compile
   Mosaic quickly, forget about them for the time being -- you can
   always recompile later.

   The DTM library is in subdirectory libdtm.  The HDF library must be
   obtained separately from ftp.ncsa.uiuc.edu in /HDF; get version
   3.3r1 or later.  Set the various options in the Makefile to point
   to all the right places, and you should be set.


After You Have Compiled
-----------------------

If you do not normally run Motif on your system (e.g., if you run Sun
OpenWindows instead), then you may get a whole bunch of run-time
errors about translations when you start Mosaic.

If this happens, copy the file XKeysymDB (included in this directory)
to /usr/lib/X11.  (If you compile Mosaic yourself, you may need to
place this file elsewhere, depending on your X configuration.)  See
the FAQ list online for more information.


X Defaults
----------

NCSA Mosaic includes sets of fallback X resources that provide
reasonable screen display properties for three configurations: color,
monochrome, and color SGI.  (Color SGI has its own configuration since
SGI systems commonly use a gamma correction factor of 1.7, which makes
their screens brighter than usual.)

If you compile NCSA Mosaic out of the box, or if you download a binary
from ftp.ncsa.uiuc.edu, the default resources will be for a color
display (or, if you compile on an SGI, the default will be for color
SGI).  See the Makefile for information on how to have monochrome
resources by default.  On the command line, the flags '-mono' and
'-color' allow you to switch resource configurations at runtime.

For your convenience, three corresponding X app-defaults files are
included in this distribution: app-defaults.color,
app-defaults.color-sgi, and app-defaults.mono.


Bug Reports and Comments
------------------------

Bug reports and other comments can be sent to mosaic-x@ncsa.uiuc.edu.

If you find NCSA Mosaic useful or particularly interesting, please
also send us a note -- continued development of this project partially
depends on user feedback and support.


Cheers,

Marc Andreessen and Eric Bina
Software Development Group
National Center for Supercomputing Applications
University of Illinois at Urbana-Champaign
marca@ncsa.uiuc.edu, ebina@ncsa.uiuc.edu
