#' @import mosaicCore
#' @export 
mosaicCore::makeFun

#' @import ggformula
#' @import ggplot2

NA

#' mosaic
#'
#' Data sets and utilities from Project MOSAIC (mosaic-web.org) used to teach mathematics, 
#' statistics, computation and modeling.  Funded by the NSF, Project MOSAIC is a 
#' community of educators working to tie together aspects of quantitative work that students 
#' in science, technology, engineering and mathematics will need in their professional lives, 
#' but which are usually taught in isolation, if at all.
#'
#'
#' @name mosaic-package
#' @aliases mosaic mosaic-package
#' @title mosaic: the Project MOSAIC package
#' @author Randall Pruim (\email{rpruim@@calvin.edu}), Daniel Kaplan (\email{kaplan@@macalester.edu}), Nicholas Horton (\email{nhorton@@smith.edu})
#' @references
#' \url{http://www.mosaic-web.org}
#'
#' @keywords package
#' @importFrom Matrix mean
#' @importFrom utils head modifyList
# #' @importFrom gridExtra grid.arrange arrangeGrob
# #' @importFrom ggdendro segment label dendro_data
#' @import methods
#' @importFrom MASS fitdistr fractions
#' @importFrom dplyr bind_rows bind_cols 
#' @importFrom dplyr filter select mutate summarise group_by arrange ungroup 
#' @importFrom tibble tibble rownames_to_column
#' @importFrom dplyr do n first left_join 
#' @importFrom tidyr gather
#' @importFrom grid grid.polyline grid.text grid.lines grid.points grid.rect grid.layout gpar unit.c 
#' @import lattice
#' @import mosaicData
#' @importFrom grDevices col2rgb colorRampPalette gray heat.colors rgb topo.colors
#' @importFrom graphics hist plot stem
#' @importFrom stats TukeyHSD nls
#' @importFrom stats df
#' @importFrom stats addmargins aggregate anova aov approxfun as.formula chisq.test coef
#' @importFrom stats complete.cases confint cutree dbeta dcauchy dchisq dexp dgamma dist dlnorm dnorm
#' @importFrom stats dpois dt dweibull formula lm loess loess.control lsfit model.frame model.matrix
#' @importFrom stats  model.response na.omit pnorm ppoints predict printCoefmat qnorm qt rbinom resid
#' @importFrom stats rmultinom rnorm runif setNames splinefun terms uniroot update vcov wilcox.test xtabs
#' @importFrom utils browseURL capture.output menu read.csv read.table str tail head

"_PACKAGE"
