#' @rdname defunct
#'
#' @export

gwm <- function(...) { # formula, data = parent.frame(), drop = FALSE, ...) {
  .Defunct(msg = "gwm() has been removed from `mosaic'.  \n    It will be replaced by better tools in `mosaicModel'.")
}




