#' Defunct functions now in the fetch package
#' 
#' These functions have been moved to the fetch package.
#'
#' @param ... arguments 
#' @rdname defunct-fetch
#' 
#' @export

fetchData <- function(...) {
  .Defunct(msg="Use fetchData() from the `fetch' package instead.")
}
