#' Defunct functions
#' 
#' The following functions were once a part of the mosaic pacakge but have
#' been removed.  In some cases, an alternative is available and is suggested
#' if you attempt to execute the function.
#' 
#' @rdname defunct
#' @param ... arguments, ignored since the function is defunct
#' @export

perctable <- function(...) 
{
  .Defunct("tally")
}

#' @rdname defunct
#' @export

proptable <- function(...) 
{
  .Defunct("tally")
}

