#' Sleep and Memory
#' 
#' Sleep and Memory
#' 
#' @docType data
#' @name Sleep
#' @usage data(Sleep)
#' @format  A data.frame with 24 observations on the following 2 variables.
#' \describe{
#'    \item{\code{Group }}{treatment group of the subject}
#'    \item{\code{Words }}{number of words recalled}
#' }
#' @source These data were used in a "resampling bake-off" hosted by Robin Lock.
#'
#' @details
#' In an experiment on memory (Mednicj et al, 2008), students were given lists
#' of 24 words to memorize.  After hearing the words they were assigned at
#' random to different groups. One group of 12 students took a nap for 1.5 hours
#' while a second group of 12 students stayed awake and was given a caffeine
#' pill.  The data set records the number of words each participant was
#' able to recall after the break.  

"Sleep"