---
title: "Resources Related to the mosaic Package"
author: "Randall Pruim, Daniel T. Kaplan, and Nicholas J. Horton"
date: "`r Sys.Date()`"

  
vignette: >
  %\VignetteIndexEntry{Resources Related to the mosaic Package}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---



This vignette describes related resources and materials useful for teaching
statistics with a focus on modeling and computation.

## Package Vignettes

The `mosaic` package includes a number of vignettes. These are available 
from within R, from 
[cran.r-project.org/package=mosaic](https://cran.r-project.org/package=mosaic),
or from [www.mosaic-web.org/mosaic/](https://www.mosaic-web.org/mosaic/).

 * *Minimal R* describes a minimal set of R commands for use in 
	Introductory Statistics and discusses why it is important
		to keep the set of commands small;
		
 * *Resampling methods in R* demonstrates how to use the `mosaic`
	package to compute p-values for randomization tests and bootstrap confidence 
	intervals in a number of common situations. The examples are based on the 
	``resampling bake off'' at USCOTS 2011.

 * *ggformula/lattice conversion examples* compares the lattice and ggformula formula interfaces for creating graphics. 

 * *Less Volume, More Creativity*,
based on slides from an ICOTS 2014 workshop, introduces the `mosaic` package and
related tools and describes some of the philosophy behind the design choices
made in the `mosaic` package.
	
 * *Graphics with the mosaic package*
 is gallery of plots made using tools from the `mosaic` package.

## Auxiliary packages

Some features of the mosaic package are provided through auxiliary packages. These include:

* mosaicModel -- implements high-level systems for working with statistical models: effect-size calculation, bootstrapped confidence intervals, prediction error, graphics for models with multiple inputs. The package contains an introductory vignette.
* mosaicCalc -- provides the calculus components of mosaic, including integration, differentiation, and differential equation solving. 

<!-- See *[Modeling-based calculus with R/mosaic](http://www.mosaic-web.org/wp-content/uploads/2019/01/UMAP-mosaic-calculus.pdf)* for an instructor-oriented introduction and *[Start R in Calculus](http://project-mosaic-books.com/?page_id=22)* for a student-facing guide. -->

Install these packages using `install.packages(c("mosaicCalc", "mosaicModel"))`.

## Mosaic paper

Pruim R, Kaplan DT and Horton NJ (2017). The mosaic Package: Helping
Students to 'Think with Data' Using R. *The R Journal*, 9(1), pp.
77-102.  https://journal.r-project.org/articles/RJ-2017-024/index.html.

Abstract: The mosaic package provides a simplified and systematic introduction to the core functionality related to descriptive statistics, visualization, modeling, and simulation-based inference required in first and second courses in statistics. This introduction to the package describes some of the guiding principles behind the design of the package and provides illustrative examples of several of the most important functions it implements. These can be combined to help students 'think with data' using R in their early course work, starting with simple, yet powerful, declarative commands.

## Project MOSAIC Little Books

The following longer documents are available at 
[github.com/ProjectMOSAIC/LittleBooks](https://github.com/ProjectMOSAIC/LittleBooks/blob/master/README.md).

 * *Start Teaching Statistics Using R* includes 
some strategies for teaching beginners, and introduction to the `mosaic` package,
and some additional things that instructors should know about using R.
(A spanish language translation can be found at
https://github.com/fjaraavilaa/MOSAIC-LittleBooks-Spanish.)

 * *[A Student's Guide to R](https://github.com/ProjectMOSAIC/LittleBooks/raw/master/StudentGuide/MOSAIC-StudentGuide.pdf)* provides a brief introduction to 
the R commands needed for all the basic statistical procedures in 
an Intro Stats course.  
(A spanish language translation can be found at
https://github.com/fjaraavilaa/MOSAIC-LittleBooks-Spanish.)


<!--  * *[Start R in Calculus](http://project-mosaic-books.com/?page_id=22)* highlights features  -->
<!-- of R and the `mosaic` package that can be used to teach  -->
<!-- calculus with R. -->

 
## Textbook Related

 * *Statistical Modeling: A Fresh Approach* (DT Kaplan, second edition)]
is an introduction to statistics embracing a modeling approach and employing 
resampling methods. 
The `mosaic` package is used throughout.

    * [www.mosaic-web.org/StatisticalModeling](https://dtkaplan.github.io/SM2-bookdown/)

 * *Foundations and Applications of Statistics: An Introduction Using R* (R Pruim, second edition) 
is an R-infused probability and mathematical statistics text that emphasizes
connections between probability and statistics. The first edition of the book 
predates the `mosaic` package, but much of the code originally in the `fastR` 
package has been moved into the `mosaic` package. 
The second edition is supported by the `fastR2` package and uses `ggformula` for
plotting.
    * [https://bookstore.ams.org/amstext-28](https://bookstore.ams.org/amstext-28)
    * [www.ams.org/publications/authors/books/postpub/amstext-28](http://www.ams.org/publications/authors/books/postpub/amstext-28)

 * *Modern Data Science with R* (BS Baumer, DT Kaplan, and NJ Horton)
		is an R-infused data science text that emphasizes conceptual understanding and computation.

    * [http://mdsr-book.github.io/](https://mdsr-book.github.io/)
    
 * *The Statistical Sleuth in R* (NJ Horton)
		describes how to undertake analyses in R for the
		examples in the Third Edition of the
		*Statistical Sleuth: A Course in Methods of Data Analysis* (2013),
		by Fred Ramsey and Dan Schafer.

    * [nhorton.people.amherst.edu/sleuth](https://nhorton.people.amherst.edu/sleuth/)

 * *Introduction to the Practice of Statistics in R* (NJ Horton and BS Baumer)
		describes how to undertake analyses in R that are introduced as examples in 
		*Introduction to the Practice of
		Statistics*, by David Moore, George McCabe,
		and Bruce Craig. 
		
    * [9th edition: nhorton.people.amherst.edu/ips9](https://nhorton.people.amherst.edu/ips9/) 
    * [6th edition: nhorton.people.amherst.edu/ips6e](https://nhorton.people.amherst.edu/ips6e/) 

 * *Statistics: Unlocking the Power of Data* (Lock, Lock, Lock, Lock, and Lock) 
		is an introductory statistics textbook that embraces a resampling approach. 
		
    An annotated companion to the examples in the book implemented using R can be found at 
		
    * [github.com/rpruim/Lock5withR/](https://github.com/rpruim/Lock5withR/blob/master/README.md)
    
    and the `Lock5withR` R package provides all the data sets used in the text.
		
    * [lock5stat.com](https://www.lock5stat.com/) 
		

 * *Stats: Data and Models* (NJ Horton)
		describes how to undertake analyses in R for the
		examples in the Fourth Edition of 
		*Stats: Data and Models* (2015),
		by Dick de Veaux, Paul Velleman, and Dave Bock.

    * [nhorton.people.amherst.edu/sdm4](https://nhorton.people.amherst.edu/sdm4/)

 * *Intro Stats* (P Frenett and NJ Horton)
		describes how to undertake analyses in R for the
		examples in the Fourth and Fifth Editions of 
		*Intro Stats* (2013),
		by Dick de Veaux, Paul Velleman, and Dave Bock.

    * [5th edition: nhorton.people.amherst.edu/is5](https://nhorton.people.amherst.edu/is5/)
    * [4th edition: nhorton.people.amherst.edu/is4](https://nhorton.people.amherst.edu/is4/)


 * *Introduction to Statistical Investigations* (Tintle *et al*) 
		is another introductory statistics textbook that embraces a resampling approach. 
		
    An annotated companion to the examples in the book implemented using R can be found at 
		
    * [github.com/rpruim/ISIwithR/](https://github.com/rpruim/ISIwithR/blob/master/README.md)
    
    The `ISIwithR` R package provides all the data sets used in the text.
    Additional information about the book and the approach used there can be found at
		
    * [www.isi-sats.com/isi/](http://www.isi-stats.com/isi/)
		
  <!-- * [Open Intro Stats](https://www.openintro.org/) -->

  <!--   OpenIntro Stats now has versions of their labs designed for  -->
  <!--   use with the `mosaic` package.   -->
  <!--   The `mosaic` labs were adapted by  -->
  <!--   Ben Baumer and Galen Long of Smith College and updated to `ggformula` by Bonnie Lin and Nicholas Horton of Amherst College. -->

## Articles
 * GW Cobb, "The introductory statistics course: a Ptolemaic curriculum?", 
   *Technology Innovations in Statistics Education*, 2007, 1(1), 
   [escholarship.org/uc/item/6hb3k0nz](https://escholarship.org/uc/item/6hb3k0nz).

 * Fieberg JR, Vitense K, Johnson DH. 2020. Resampling-based methods for biologists. 
 PeerJ 8:e9089 <https://doi.org/10.7717/peerj.9089>
 
 * NJ Horton, BS Baumer, and H Wickham, "Teaching precursors to data science in introductory and second courses in statistics," *CHANCE*, 2015, 28(2):40-50, 
 [nhorton.people.amherst.edu/precursors](https://nhorton.people.amherst.edu/precursors/)

 * NJ Horton, and J Hardin, "Teaching the next generation of statistics students to "Think With Data": special issue on statistics and the undergraduate curriculum," *TAS*, 2015, 69(4):259-265, 
 [https://amstat.tandfonline.com/doi/full/10.1080/00031305.2015.1094283](https://amstat.tandfonline.com/doi/full/10.1080/00031305.2015.1094283)
 
 * D Nolan and D Temple Lang, "Computing in the statistics curricula", 
*The American Statistician*, 2010, 64(2), 
[www.stat.berkeley.edu/~statcur/Preprints/ComputingCurric3.pdf](https://www.stat.berkeley.edu/~statcur/Preprints/ComputingCurric3.pdf).

## Other Resources

 * *Example R Code/Graphs Using NHANES* by Amy Wagaman,
 <https://awagaman.people.amherst.edu/stat230/Stat230CodeCompilationExampleCodeUsingNHANES.pdf>
