---
title: "Untitled"
author: ""
date: ""
output: 
  html_document:
    fig_height: 3
    fig_width: 5
  pdf_document:
    fig_height: 3
    fig_width: 5
  word_document:
    fig_height: 3
    fig_width: 5
---

```{r, setup, include=FALSE}
require(mosaic)   # Load additional packages here 
knitr::opts_chunk$set(
  tidy=FALSE,     # display code as typed
  size="small")   # slightly smaller font for code
```

## Using RMarkdown


### Text

Text can be decorated with **bold** or *italics*.  It is also possible to 

* create [links](https://rmarkdown.rstudio.com/) 
* include mathematics like $e=mc^2$ or 
$$y = \beta_0 + \beta_1 x_1 + \beta_2 x_2$$

Be sure to put a space after the * when you are creating bullets and a space after # when 
creating section headers, but **not** between $ and the mathematical formulas.

### Graphics

If the code of an R chunk produces a plot, this plot can be displayed in the resulting file.
```{r}
gf_point(births ~ date, data = Births78)
```

### R output
Other forms of R output are also displayed as they are produced.
```{r}
favstats(~ births, data = Births78)
```

### Destination formats

This file can be knit to HTML, PDF, or Word.  In RStudio, just select the desired output file type
and click on `Knit HTML`, `Knit PDF`, or `Knit Word`.  Use the dropdown menu next to that to 
change the desired file type.

### Documenting file creation 

It's useful to record some information about how your file was created.

  * File creation date: `r Sys.Date()`
  * `r R.version.string`
  * R version (short form): `r getRversion()`
  * `mosaic` package version: `r packageVersion("mosaic")`
  * Additional session information
  
```{r echo=FALSE}
sessionInfo()  # could use devtools::session_info() if you prefer that
```
  