% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mosaic-package.R
\docType{package}
\name{mosaic-package}
\alias{mosaic-package}
\alias{mosaic}
\title{mosaic: the Project MOSAIC package}
\description{
mosaic
}
\details{
Data sets and utilities from Project MOSAIC (mosaic-web.org) used to teach mathematics,
statistics, computation and modeling.  Funded by the NSF, Project MOSAIC is a
community of educators working to tie together aspects of quantitative work that students
in science, technology, engineering and mathematics will need in their professional lives,
but which are usually taught in isolation, if at all.
}
\references{
\url{http://www.mosaic-web.org}
}
\seealso{
Useful links:
\itemize{
  \item \url{https://github.com/ProjectMOSAIC/mosaic}
  \item \url{https://www.mosaic-web.org/mosaic/}
  \item Report bugs at \url{https://github.com/ProjectMOSAIC/mosaic/issues}
}

}
\author{
Randall Pruim (\email{rpruim@calvin.edu}), Daniel Kaplan (\email{kaplan@macalester.edu}), Nicholas Horton (\email{nhorton@smith.edu})
}
\keyword{package}
